unit t2port;

interface
uses Windows,mystrtool,dialogs,sysutils,classes;

const ReadBufferSize=2048;
      WriteBufferSize=2048;

type TBufferType = array[0..ReadBufferSize-1] of byte;

const Com_Port_Write = 2;
      Com_Port_Read =  1;

type TComPortReadWriteThread = class (TThread)
///     private
      PortHDL: THandle;
      PortCS: TRTLCriticalSection;
      ThrType: integer;

      ReadWriteBuffer:TBufferType;
      ReadWriteBufferBytesCount:integer;

     protected
      procedure Execute; override;
     public
      constructor Create(ThreadType:integer);
     end;

type TCOMPort=object
               var ReadTHR:TComPortReadWriteThread;
               var WriteTHR:TComPortReadWriteThread;

               var tickCount:integer;
               procedure ResetTimeOut;
               function TimeOut(ticks:integer):boolean;

               function OpenPort(portName: string):THandle;
               procedure ClosePort;

               procedure SendData(myData:string);
               procedure SendDataBinary(data:TBufferType; howMany:integer);
               function GetDataLTime(howMany:integer;var data:TBufferType; eqDataStr:string; ltime:integer):boolean;
               function GetData(howMany:integer;var data:TBufferType; eqDataStr:string):boolean;
           end;




implementation

procedure TCOMPort.ResetTimeOut;
begin
  tickCount:=0;
end;

function TCOMPort.TimeOut(ticks:integer):boolean;
begin
 inc(tickCount);
 sleep(3);
 if tickCount>ticks then result:=true else result:=false;
 
end;

constructor TComPortReadWriteThread.Create(ThreadType:integer);
begin
 InitializeCriticalSection(PortCS);
 ThrType:=ThreadType;

 zeromemory(@ReadWriteBuffer,sizeof(ReadWriteBuffer));
 ReadWriteBufferBytesCount:=0;

 FreeOnTerminate := True;
 inherited Create(True);
end;

procedure TComPortReadWriteThread.Execute;
var nSent, nRead:Cardinal;
var oneByte:byte;
begin

 while not Terminated do  // ->
  begin
   EnterCriticalSection(PortCS);

   case ThrType of // <-
    Com_Port_Write: begin //***

                     if PortHDL<>INVALID_HANDLE_VALUE then
                     if ReadWriteBufferBytesCount>0 then
                       begin
                        nSent:=0;
                        WriteFile( portHDL, ReadWriteBuffer, ReadWriteBufferBytesCount, nSent, nil );

                        if nsent>0 then
                        dec(ReadWriteBufferBytesCount,nSent);

                       end;

                   end;  //***
    Com_Port_Read: begin //***

                    if portHDL<>INVALID_HANDLE_VALUE then
                       begin

                        nRead:=0;
                        ReadFile(portHDL,oneByte,1,nRead,nil);

                        if nRead=1 then
                         begin
                          inc(ReadWriteBufferBytesCount);
                          ReadWriteBuffer[ReadWriteBufferBytesCount-1]:=oneByte;
                         end;

                       end;
                   end;  //***

   end; // case <-

   LeaveCriticalSection(PortCS);
   sleep(1);
  end; // -> while not Terminated

end;

function TCOMPort.OpenPort(portName: string):THandle;
var hCommPort      : THandle;
    MyCommTimeouts : TCommTimeouts;
    MyDCB          : TDCB;
begin



 ReadThr:=TComPortReadWriteThread.Create(COM_PORT_READ);
 WriteThr:=TComPortReadWriteThread.Create(COM_PORT_WRITE);

 ReadThr.Priority:=tpHigher;
 WriteThr.Priority:=tpHigher;

 EnterCriticalSection(ReadThr.PortCS);
 EnterCriticalSection(WriteThr.PortCS);

 ReadThr.ReadWriteBufferBytesCount:=0;
 WriteThr.ReadWriteBufferBytesCount:=0;

 hCommPort := INVALID_HANDLE_VALUE;

 hCommPort := CreateFile(PChar('\\.\' + portName),
                         GENERIC_READ or GENERIC_WRITE,
                         0, // Not shared
                         nil, // No security attributes
                         OPEN_EXISTING,
                         FILE_ATTRIBUTE_NORMAL{ or FILE_FLAG_OVERLAPPED},
                         0 // No template
                         ) ;

   result:=hCommPort;


    MycommTimeouts.ReadIntervalTimeout := MAXDWORD;
    MycommTimeouts.ReadTotalTimeoutMultiplier := 0;
    MycommTimeouts.ReadTotalTimeoutConstant := 0;
    MycommTimeouts.WriteTotalTimeoutMultiplier := 0;
    MycommTimeouts.WriteTotalTimeoutConstant:=0;

    GetCommState(hCommPort,myDCB);

    MyDCB.DCBlength := SizeOf(TDCB);
    MyDCB.BaudRate  := CBR_9600;
    MyDCB.Flags     := $0001;
    MyDCB.StopBits  := ONESTOPBIT;
    MyDCB.Parity    := NOPARITY;
    MyDCB.ByteSize  := 8;

    SetCommState(hCommPort, MyDCB);
    SetCommTimeouts(hCommPort, MyCommTimeouts);

   WriteThr.PortHDL:=hCommPort;
   ReadThr.PortHDL:=hCommPort;

    PurgeComm(hCommPort, PURGE_TXCLEAR or PURGE_RXCLEAR);

   ReadTHr.Resume;
   WriteTHr.Resume;
   LeaveCriticalSection(WriteThr.PortCS);
   LeaveCriticalSection(ReadThr.PortCS);

end;


function WaitForMultipleThreads(const ThrArr : array of TThread;
                                           TimeOutVal : DWORD) : Word;
var
  Hndls : TWOHandleArray;
  I     : Integer;
begin
  //This loop takes all the values from the input array and
  //assigns them to the handle array (TWOHandleArray) to be
  //passed to WaitForMultipleObjects
  for I := 0 to High(ThrArr) do
    Hndls[I] := ThrArr[I].Handle;
  Result := WaitForMultipleObjects(High(ThrArr) + 1, @Hndls, True, TimeOutVal)
end;

procedure TComPort.ClosePort;
begin
 ReadThr.Terminate;
 WriteThr.Terminate;

 WaitForMultipleThreads([ReadTHR, WriteTHR], INFINITE);

 enterCriticalSection(WriteThr.PortCS);

 if WriteTHR.PortHDL=INVALID_HANDLE_VALUE then exit;
// PurgeComm(WriteTHR.PortHDL, PURGE_TXCLEAR or PURGE_RXCLEAR);
 CloseHandle(WriteTHR.PortHDL);

 LeaveCriticalSection(WriteThr.PortCS);

 DeleteCriticalSection(WriteThr.PortCS);
 DeleteCriticalSection(ReadThr.PortCS); 
end;

procedure TComPort.SendDataBinary(data:TBufferType; howMany:integer);
var i,k:integer;
begin

sleep(1);
 EnterCriticalSection(WriteTHR.PortCS);

 k:=howMany;

 for i := 0 to k-1 do
  WriteTHR.ReadWriteBuffer[WriteTHR.ReadWriteBufferBytesCount+i]:= data[i];

  WriteTHR.ReadWriteBufferBytesCount:=WriteTHR.ReadWriteBufferBytesCount+k;

 LeaveCriticalSection(WriteTHR.PortCS);

end;

procedure TComPort.SendData(myData:string);
var i,k:integer;
begin


sleep(1);
 EnterCriticalSection(WriteTHR.PortCS);

 k:=getWordCount(myData);


 for i := 0 to k-1 do
  WriteTHR.ReadWriteBuffer[WriteTHR.ReadWriteBufferBytesCount+i]:= HexToInt(getWord(myData,i+1));

  WriteTHR.ReadWriteBufferBytesCount:=WriteTHR.ReadWriteBufferBytesCount+k;

 LeaveCriticalSection(WriteTHR.PortCS);
end;

function Match(buf1,buf2:TBufferType; len1,len2:integer ):boolean;
var i,j:integer;
var match:boolean;
begin

result:=false;


for i := 0 to len1 - len2 do
 begin

 match:=true;

 for j := 0 to len2 - 1 do
  begin
    if buf1[j+i]<>buf2[j] then match:=false;
  end;

 result:=result or match;

 end;


end;

function TComPort.GetDataLTime(howMany:integer;var data:TBufferType; eqDataStr:string; ltime:integer):boolean;
begin
   ResetTimeOut;
   repeat
    if TimeOut(ltime) then break;
   until GetData(howMany,data,eqDataStr);

   if TimeOut(ltime) then result:=false
    else result:=true;
end;

function TComPort.GetData(howMany:integer;var data:TBufferType; eqDataStr:string):boolean;
var i:integer;
eqData:TBufferType;
eqMany:integer;
begin


sleep(1);

eqMany:=getWordCount(eqDataStr);


 for i := 1 to eqMany do
   eqData[i-1]:= HexToInt(getWord(eqDataStr,i));

  result:=true;


EnterCriticalSection(ReadTHR.PortCS);
  if ReadTHR.ReadWriteBufferBytesCount<howMany then result:=false;
LeaveCriticalSection(ReadTHR.PortCS);

  if result=false then exit;


EnterCriticalSection(ReadTHR.PortCS);


    data:=ReadTHR.ReadWriteBuffer;

   result:=Match(data,eqData,
                 ReadTHR.ReadWriteBufferBytesCount,
                 eqMany);

   if result=true then
    begin
    for i:= howmany to ReadTHR.readWriteBufferBytesCount-1 do
      ReadTHR.readWriteBuffer[i-howMany]:=ReadTHR.readWriteBuffer[i];
      ReadTHR.readWriteBufferBytesCount:=ReadTHR.readWriteBufferBytesCount - howMany;

// DELETE ALL IF SUCCEED    readBufferBytes:=0{?}

        ReadTHR.readWriteBufferBytesCount:=0;

    end;
LeaveCriticalSection(ReadTHR.PortCS);

end;




end.
