unit t2video;

interface

uses DirectShow9, ActiveX, Windows, Classes,graphics,dialogs;

type TDeviceLink = record
                   deviceString : String;
                   deviceMoniker : IMoniker;
                  end;

type TDeviceLinks = array of TDeviceLink;

var  g_pGraph             : IGraphBuilder;
     g_pCapture           : ICaptureGraphBuilder2;

     g_pMC                : IMediaControl;
     g_pVW                : IVideoWindow;

     pSrcFilter           : IBaseFilter;
     pGrabberFilter       : IBaseFilter;

     pGrabber:ISampleGrabber;
     media_type: TAMMEDIATYPE;
     pVih : pVIDEOINFOHEADER;

     gChannels,
             gWidth,gHeight:integer;


function DeviceLinksToStringList(DeviceLinks:TDeviceLinks):TStringList;
function EnumerateDevices(GUID_type:TGUID):TDeviceLinks;
function DeviceNameToMoniker(DeviceName:String;DeviceLinks:TDeviceLinks):IMoniker;

procedure chooseVideoDevice( pmVideo: IMoniker; zWindowHandle:HWND; zClientRect:TRect);


  function GetBitmap1(Bitmap: TBitmap): Boolean;

implementation

  function GetDIBLineSize(BitCount, Width: Integer): Integer;
  begin
    if BitCount = 15 then
      BitCount := 16;
    Result := ((BitCount * Width + 31) div 32) * 4;
  end;

  procedure FreeMediaType(mt: PAMMediaType);
    begin
    if (mt^.cbFormat <> 0) then
    begin
      CoTaskMemFree(mt^.pbFormat);
      // Strictly unnecessary but tidier
      mt^.cbFormat := 0;
      mt^.pbFormat := nil;
    end;
    if (mt^.pUnk <> nil) then mt^.pUnk := nil;
  end;


  function GetBitmap(Bitmap: TBitmap; Buffer: Pointer; BufferLen: Integer): Boolean;
  var
    hr: HRESULT;
    BIHeaderPtr: PBitmapInfoHeader;
    MediaType: TAMMediaType;
    BitmapHandle: HBitmap;
    DIBPtr: Pointer;
    DIBSize: LongInt;
  begin
    Result := False;
    if not Assigned(Bitmap) then
      Exit;
    if Assigned(Buffer) and (BufferLen = 0) then
      Exit;
    hr := pGrabber.GetConnectedMediaType(MediaType);
    if hr <> S_OK then
      Exit;
    try
      if IsEqualGUID(MediaType.majortype, MEDIATYPE_Video) then
      begin
        BIHeaderPtr := Nil;
        if IsEqualGUID(MediaType.formattype, FORMAT_VideoInfo) then
        begin
          if MediaType.cbFormat = SizeOf(TVideoInfoHeader) then  // check size
            BIHeaderPtr := @(PVideoInfoHeader(MediaType.pbFormat)^.bmiHeader);
        end
        else if IsEqualGUID(MediaType.formattype, FORMAT_VideoInfo2) then
        begin
          if MediaType.cbFormat = SizeOf(TVideoInfoHeader2) then  // check size
            BIHeaderPtr := @(PVideoInfoHeader2(MediaType.pbFormat)^.bmiHeader);
        end;
        // check, whether format is supported by TSampleGrabber
        if not Assigned(BIHeaderPtr) then
          Exit;
        BitmapHandle := CreateDIBSection(0, PBitmapInfo(BIHeaderPtr)^,
                                         DIB_RGB_COLORS, DIBPtr, 0, 0);
        if BitmapHandle <> 0 then
        begin
          try
            if DIBPtr = Nil then
              Exit;
            // get DIB size
            DIBSize := BIHeaderPtr^.biSizeImage;
            if DIBSize = 0 then
            begin
              with BIHeaderPtr^ do
                DIBSize := GetDIBLineSize(biBitCount, biWidth) * biHeight * biPlanes;
            end;
            // copy DIB
            if not Assigned(Buffer) then
            begin
              // get buffer size
              BufferLen := 0;
              hr := pGrabber.GetCurrentBuffer(BufferLen, Nil);
              if (hr <> S_OK) or (BufferLen <= 0) then
                Exit;
              // copy buffer to DIB
              if BufferLen > DIBSize then  // copy Min(BufferLen, DIBSize)
                BufferLen := DIBSize;
              hr := pGrabber.GetCurrentBuffer(BufferLen, DIBPtr);
              if hr <> S_OK then
                Exit;
            end
            else
            begin
              if BufferLen > DIBSize then  // copy Min(BufferLen, DIBSize)
                BufferLen := DIBSize;
              Move(Buffer^, DIBPtr^, BufferLen);
            end;
            Bitmap.Handle := BitmapHandle;
            Result := True;
          finally
            if Bitmap.Handle <> BitmapHandle then  // preserve for any changes in Graphics.pas
              DeleteObject(BitmapHandle);
          end;
        end;
      end;
    finally
      FreeMediaType(@MediaType);
    end;
  end;


  function GetBitmap1(Bitmap: TBitmap): Boolean;
  begin
    Result := GetBitmap(Bitmap, Nil, 0);
  end;


function DeviceLinksToStringList(DeviceLinks:TDeviceLinks):TStringList;
var i:integer;
begin
 result:=TStringList.Create;
 for i := 0 to length(DeviceLinks)-1 do
  begin
    result.Add(DeviceLinks[i].deviceString)
  end;
end;

function DeviceNameToMoniker(DeviceName:String;DeviceLinks:TDeviceLinks):IMoniker;
var i:integer;
begin
 for i := 0 to length(DeviceLinks) - 1 do
  if DeviceLinks[i].deviceString=DeviceName then result:=DeviceLinks[i].deviceMoniker;

end;

function EnumerateDevices(GUID_type:TGUID):TDeviceLinks;
var  hr : HRESULT;
     pCreateDevEnum : ICreateDevEnum;
     pEm : IEnumMoniker;
     uIndex : UINT;
     pM : IMoniker;
     cFetched : cardinal;
     pBag : IPropertyBag;
     Name : OLEVARIANT;

     iii:integer;
begin


//  result:=TStringList.Create;
  setlength(result,0);

  hr := CoCreateInstance(CLSID_SystemDeviceEnum,
                         Nil,
                         CLSCTX_INPROC_SERVER,
                         IID_ICreateDevEnum,
                         pCreateDevEnum);

  if(hr <> NOERROR) then exit;

  hr := pCreateDevEnum.CreateClassEnumerator(GUID_type,
                                             pEm,
                                             0);
  if(hr <> NOERROR) then exit;

  pEm.Reset;

  uIndex := 0;

  while (pEm.Next(1, pM, @cFetched) = S_OK) do
  Begin
    hr := pM.BindToStorage(Nil, Nil, IPropertyBag, pBag);
    if(SUCCEEDED(hr)) then
    Begin
      hr := pBag.Read('FriendlyName', Name, Nil);

      if (hr = NOERROR) then
       begin
        iii:=length(result);
        setlength(result,iii+1);
        result[iii].deviceString:=Name;
        result[iii].deviceMoniker:=pM;
       end;


      (*
      if (hr = NOERROR) then
       result.Add(Name);

          deviceMoniker:=pM;
       if Name='U200 Video' then
        begin
          deviceMoniker:=pM;
        end;
      *)


      pBag := Nil;
    End;
    pM := Nil;
    inc(uIndex);
  End;
  pEm := Nil;

end;



procedure chooseVideoDevice( pmVideo: IMoniker; zWindowHandle:HWND; zClientRect:TRect);
var   hr : HResult;
  rc : TRect;
    cy, cyBorder : Integer;
Begin

// >> GETINTERFACES
  // Create the filter graph
  hr := CoCreateInstance(CLSID_FilterGraph, Nil, CLSCTX_INPROC,
        IID_IGraphBuilder, g_pGraph);



  // Create the capture graph builder
  hr := CoCreateInstance(CLSID_CaptureGraphBuilder2,
        Nil, CLSCTX_INPROC, IID_ICaptureGraphBuilder2,
        g_pCapture);


  // Obtain interfaces for media control and Video Window
  hr := g_pGraph.QueryInterface(IID_IMediaControl, g_pMC);


//  hr := g_pMC.Run;


// net:  IID_IMediaEvent, notifywindow


  hr := g_pGraph.QueryInterface(IID_IVideoWindow, g_pVW);



// <<<<<<<<<<<<<<<

  hr := g_pCapture.SetFiltergraph(g_pGraph);


// FindCaptureDevice
  hr := pmVideo.BindToObject(Nil, Nil, IID_IBaseFilter, pSrcFilter);
//

  hr := g_pGraph.AddFilter(pSrcFilter, 'Video Capture');



// ADDsampleg

    // Create the Sample Grabber.
  hr := Cocreateinstance(CLSID_SampleGrabber, nil, CLSCTX_INPROC,
                         IID_IBASEFilter, pGrabberFilter);
  hr := g_pGraph.AddFilter(pGrabberFilter, 'Sample Grabber');

  hr := pGrabberFilter.QueryInterface(IID_ISampleGrabber,pGrabber);


//

// setmediatype
  ZeroMemory(@media_type, sizeof(AM_MEDIA_TYPE));
  media_type.majortype:= MEDIATYPE_Video;
  media_type.subtype:= MEDIASUBTYPE_RGB24;
  hr:= pGrabber.SetMediaType(media_type);

//


  hr:= g_pCapture.RenderStream(@PIN_CATEGORY_PREVIEW,
                              @MEDIATYPE_Video,
                              pSrcFilter,
                              pGrabberFilter,
                              nil);

// sgGetSampleGrabberMediaType
  ZeroMemory(@media_type, sizeof(AM_MEDIA_TYPE));
 hr:=pGrabber.GetConnectedMediaType(media_type);


 pVih:= media_type.pbFormat;
        gChannels:= pVih.bmiHeader.biBitCount div 8;
        gWidth:= pVih.bmiHeader.biWidth;
        gHeight:= pVih.bmiHeader.biHeight;

//Form1.ListBox1.Items.add(intToStr(gChannels));
//Form1.ListBox1.Items.add(intToStr(gWidth));
//Form1.ListBox1.Items.add(intToStr(gHeight));



//


// setupvideowindow


   g_pVW.put_Owner(zWindowHandle);    // We own the window now
   g_pVW.put_WindowStyle(WS_CHILD);    // you are now a child

//   resizevideowindow

        rc := zClientRect;
        cyBorder := GetSystemMetrics(SM_CYBORDER);
        cy := cyBorder;
        rc.bottom := rc.Bottom - cy;
        g_pVW.SetWindowPosition(0, 0, rc.right, rc.bottom); // be this big

        g_pVW.put_Visible(TRUE);

//




  hr:= pGrabber.SetOneShot(FALSE);
  hr:= pGrabber.SetBufferSamples(TRUE);


g_pMC.run;


End;




end.
